-----------------------------------------------------------------------------------
-- Create the CASE_SUMMARY table to replace the default view                       
-----------------------------------------------------------------------------------
DROP TABLE CAUSER.T_CFREPORTING_CASE_SUMMARY;
CREATE TABLE CAUSER.T_CFREPORTING_CASE_SUMMARY (
		"DMCASE_KEY" BIGINT NOT NULL,
		"DMUSER_KEY" INTEGER NOT NULL,
		"DMCASETYPE_KEY" INTEGER NOT NULL,
		"TSCREATIONTIME" TIMESTAMP,
		"OUTGOING" INTEGER NOT NULL,
		"TIMEUSAGE" REAL NOT NULL,
		"RESOLUTION_TIMEUSAGE" BIGINT,
		"TIMEINTERVAL" TIMESTAMP NOT NULL,
		"VMAE_CFREPORTING_CURRENTLOSS" REAL,
		"VMAE_CFREPORTING_LOSSRECOVERY" REAL,
		"VMAE_CFREPORTING_RESOLUTION_KEY" INTEGER,
		"VMAE_CFREPORTING_RESOLUTIONDATE_KEY" INTEGER,
		"VMAE_CFREPORTING_CASEOWNER_KEY" INTEGER,
		"VMAE_CFREPORTING_SUPERVISOR_KEY" INTEGER,
		"VMAE_CFREPORTING_CREATEDATE_KEY" INTEGER,
		"VMAE_CFREPORTING_LOSSEXPOSURE" REAL,
		"VMAE_CFREPORTING_STATUS_KEY" INTEGER
	)
	ORGANIZE BY ROW
	DATA CAPTURE NONE
	IN "TARGET_DATA_TS"
	COMPRESS NO;
-----------------------------------------------------------------------------------
-- Create indexes for the new CASE_SUMMARY table                                 --
-----------------------------------------------------------------------------------
CREATE INDEX CAUSER.CASE_SUMMARY_DMCASE_KEY_IDX ON CAUSER.T_CFREPORTING_CASE_SUMMARY (DMCASE_KEY ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
CREATE INDEX CAUSER.CASE_SUMMARY_OWNER_KEY_IDX ON CAUSER.T_CFREPORTING_CASE_SUMMARY (VMAE_CFREPORTING_CASEOWNER_KEY ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
CREATE INDEX CAUSER.CASE_SUMMARY_CREATEDATE_KEY_IDX ON CAUSER.T_CFREPORTING_CASE_SUMMARY (VMAE_CFREPORTING_CREATEDATE_KEY ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
CREATE INDEX CAUSER.CASE_SUMMARY_RESOLUTIONDATE_KEY_IDX ON CAUSER.T_CFREPORTING_CASE_SUMMARY (VMAE_CFREPORTING_RESOLUTIONDATE_KEY ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
CREATE INDEX CAUSER.CASE_SUMMARY_STATUS_KEY_IDX ON CAUSER.T_CFREPORTING_CASE_SUMMARY (VMAE_CFREPORTING_STATUS_KEY ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
-----------------------------------------------------------------------------------
-- Create CASE_SUMMARY_AUDIT table to track statistics each time the refresh     --
--     procedure is called.                                                      --
-----------------------------------------------------------------------------------
DROP TABLE CAUSER.CFREPORTING_CASE_SUMMARY_AUDIT;
CREATE TABLE CAUSER.CFREPORTING_CASE_SUMMARY_AUDIT (
		"ID" BIGINT NOT NULL GENERATED BY DEFAULT AS IDENTITY ( START WITH 1 INCREMENT BY 1 MINVALUE -9223372036854775808 MAXVALUE 9223372036854775807 NO CYCLE NO CACHE NO ORDER ), 
		"ROWS_READ" BIGINT, 
		"ROWS_SKIPPED" BIGINT, 
		"ROWS_LOADED" BIGINT, 
		"ROWS_REJECTED" BIGINT, 
		"ROWS_DELETED" BIGINT, 
		"ROWS_COMMITTED" BIGINT, 
		"CALLER" VARCHAR(50),
		"UPDATE_TIME" TIMESTAMP NOT NULL GENERATED ALWAYS FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP , 
		"MSG_RETRIEVAL" VARCHAR(500),
		"MSG_REMOVAL" VARCHAR(500)
	)
	ORGANIZE BY ROW
	DATA CAPTURE NONE 
	IN "TARGET_DATA_TS"
	COMPRESS YES;
-----------------------------------------------------------------------------------
-- Create a stored procedure to refresh the CASE_SUMMARY table                   --
-----------------------------------------------------------------------------------
--#SET TERMINATOR @
CREATE OR replace PROCEDURE CAUSER.REFRESH_CASE_SUMMARY_TABLE
language sql
L1: begin
   DECLARE stmt VARCHAR(20000);
   DECLARE rows_read             INTEGER;
   DECLARE rows_skipped          INTEGER;
   DECLARE rows_loaded           INTEGER;
   DECLARE rows_rejected         INTEGER;
   DECLARE rows_deleted          INTEGER;
   DECLARE rows_committed        INTEGER;
   DECLARE rows_partitioned      INTEGER;
   DECLARE num_agentinfo_entries INTEGER;
   DECLARE msg_retrieval         VARCHAR(500);
   DECLARE msg_removal           VARCHAR(500);
   DECLARE res RESULT_SET_LOCATOR varying;
   
   set stmt = '(SELECT f.dmcase_key, 
       f.dmuser_key, 
       f.dmcasetype_key, 
       (timeinterval - CAST(timeusage AS BIGINT) MINUTES) AS TSCREATIONTIME, 
       f.outgoing, 
       f.timeusage, 
       Timestampdiff(4, Coalesce(d1.cfreporting_resolutiondate, f.timeinterval) 
                        -Coalesce(d2.cfreporting_createdate, (f.timeinterval - CAST(timeusage AS BIGINT) MINUTES))) AS RESOLUTION_TIMEUSAGE, 
       f.timeinterval, 
       f.vmae_cfreporting_currentloss, 
       f.vmae_cfreporting_lossrecovery, 
       f.vmae_cfreporting_resolution_key, 
       f.vmae_cfreporting_resolutiondate_key, 
       f.vmae_cfreporting_caseowner_key, 
       f.vmae_cfreporting_supervisor_key, 
       f.vmae_cfreporting_createdate_key, 
       f.vmae_cfreporting_lossexposure, 
       f.vmae_cfreporting_status_key 
FROM   causer.f_dmcaseload f, 
       causer.x_processedstate x, 
       causer.d_dmdatafield_cfreporting_resolutiondate d1, 
       causer.d_dmdatafield_cfreporting_createdate d2 
WHERE  f.outgoing > 0 
       AND f.vmae_cfreporting_resolutiondate_key = d1.cfreporting_resolutiondate_key 
       AND f.vmae_cfreporting_createdate_key = d2.cfreporting_createdate_key 
       AND f.pajobid < x.jobid 

UNION

SELECT f.dmcase_key, 
       f.dmuser_key, 
       f.dmcasetype_key, 
       f.tscreationtime, 
       CASE 
         WHEN s.cfreporting_status = ''Completed''
		 THEN 1 
         ELSE 0 
       END AS OUTGOING, 
       CASE 
         WHEN s.cfreporting_status = ''Completed'' 
         THEN Timestampdiff(4, d1.cfreporting_resolutiondate - f.tscreationtime) 
         ELSE Timestampdiff(4, CURRENT TIMESTAMP - f.tscreationtime) 
       END AS TIMEUSAGE, 
       Timestampdiff(4, Coalesce(d1.cfreporting_resolutiondate, CURRENT TIMESTAMP) - 
                        Coalesce(d2.cfreporting_createdate, tscreationtime)) AS RESOLUTION_TIMEUSAGE, 
       CURRENT TIMESTAMP AS TIME_INTERVAL, 
       f.vmae_cfreporting_currentloss, 
       f.vmae_cfreporting_lossrecovery, 
       f.vmae_cfreporting_resolution_key, 
       f.vmae_cfreporting_resolutiondate_key, 
       f.vmae_cfreporting_caseowner_key, 
       f.vmae_cfreporting_supervisor_key, 
       f.vmae_cfreporting_createdate_key, 
       f.vmae_cfreporting_lossexposure, 
       f.vmae_cfreporting_status_key 
FROM   causer.f_dmcasewip f, 
       causer.d_dmdatafield_cfreporting_status s, 
       causer.d_dmdatafield_cfreporting_resolutiondate d1, 
       causer.d_dmdatafield_cfreporting_createdate d2 
WHERE  f.vmae_cfreporting_status_key = s.cfreporting_status_key 
       AND f.vmae_cfreporting_resolutiondate_key = d1.cfreporting_resolutiondate_key 
       AND f.vmae_cfreporting_createdate_key = d2.cfreporting_createdate_key 
)';
   
   CALL sysproc.admin_cmd( 'LOAD FROM ' || stmt || ' OF CURSOR MESSAGES ON SERVER REPLACE INTO CAUSER.T_CFREPORTING_CASE_SUMMARY
                            NONRECOVERABLE INDEXING MODE REBUILD ALLOW NO ACCESS' );
   ASSOCIATE RESULT SET locators(res) WITH PROCEDURE sysproc.admin_cmd;
   ALLOCATE rscur CURSOR FOR RESULT SET res;
   FETCH rscur INTO rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,rows_partitioned,num_agentinfo_entries,
                    msg_retrieval,msg_removal;
   insert into causer.cfreporting_case_summary_audit (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,caller,msg_retrieval,msg_removal) values
                  (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,'causer.refresh_case_summary_table',msg_retrieval,msg_removal);
END l1@
--#SET TERMINATOR ;
-----------------------------------------------------------------------------------
-- Initiate the first refresh of the CASE_SUMMARY table using the procedure      --
--     named CAUSER.REFRESH_CASE_SUMMARY_TABLE                                   --
-----------------------------------------------------------------------------------
CALL CAUSER.REFRESH_CASE_SUMMARY_TABLE;
-----------------------------------------------------------------------------------
-- Update the statistics for the CAUSER.T_CASE_SUMMARY_TABLE                     --
-----------------------------------------------------------------------------------
RUNSTATS ON TABLE CAUSER.T_CFREPORTING_CASE_SUMMARY ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
-----------------------------------------------------------------------------------
-- Redefine the CASE_SUMMARY view                                                --
-----------------------------------------------------------------------------------
DROP VIEW CAUSER.CFREPORTING_CASE_SUMMARY;
CREATE VIEW CAUSER.CFREPORTING_CASE_SUMMARY (DMCASE_KEY, DMUSER_KEY, DMCASETYPE_KEY, TSCREATIONTIME, OUTGOING, TIMEUSAGE, RESOLUTION_TIMEUSAGE, TIMEINTERVAL, VMAE_CFREPORTING_CURRENTLOSS, VMAE_CFREPORTING_LOSSRECOVERY, VMAE_CFREPORTING_RESOLUTION_KEY, VMAE_CFREPORTING_RESOLUTIONDATE_KEY, VMAE_CFREPORTING_CASEOWNER_KEY, VMAE_CFREPORTING_SUPERVISOR_KEY, VMAE_CFREPORTING_CREATEDATE_KEY, VMAE_CFREPORTING_LOSSEXPOSURE, VMAE_CFREPORTING_STATUS_KEY) AS SELECT * FROM CAUSER.T_CFREPORTING_CASE_SUMMARY;
ALTER VIEW CAUSER.CFREPORTING_CASE_SUMMARY ENABLE QUERY OPTIMIZATION;
GRANT CONTROL ON TABLE CAUSER.CFREPORTING_CASE_SUMMARY TO USER DSRDBM01;
GRANT SELECT ON TABLE CAUSER.CFREPORTING_CASE_SUMMARY TO USER DSRDBM01 WITH GRANT OPTION;
-----------------------------------------------------------------------------------
-- Define scheduled tasks to refresh the CASE_SUMMARY table                      --
-----------------------------------------------------------------------------------
--
CREATE USER TEMPORARY TABLESPACE "SYSTOOLSTMPSPACE"
	IN DATABASE PARTITION GROUP "IBMCATGROUP"
	PAGESIZE 32 K
	MANAGED BY AUTOMATIC STORAGE
	EXTENTSIZE 4
	BUFFERPOOL "IBMDEFAULTBP"
	OVERHEAD INHERIT
	TRANSFERRATE INHERIT
	USING STOGROUP "IBMSTOGROUP"
	FILE SYSTEM CACHING;
--
GRANT USE OF TABLESPACE "SYSTOOLSTMPSPACE" TO PUBLIC;
--
-----------------------------------------------------------------------------------
-- Define refresh of summary table to run hourly                                 --
-----------------------------------------------------------------------------------
--CALL SYSPROC.ADMIN_TASK_REMOVE('REFRESH CF REPORTING CASE SUMMARY',NULL);
--
CALL SYSPROC.ADMIN_TASK_ADD('REFRESH CF REPORTING CASE SUMMARY',CURRENT_TIMESTAMP,NULL,NULL,'0 * * * *','CAUSER','REFRESH_CASE_SUMMARY_TABLE',NULL,NULL,NULL);
--